############################### Blonkers! ##############################
# File: game.py
#
# Imported by: __main__.py
#
# Description: Contains the Screen class, which handles the creation and
#   updating of the main display window.
#
#
# (C) Copyright 2011 by Chris Flesner
# This file is released under a GNU All-Permissive License:
# Copying and distribution of this file, with or without modification,
# are permitted in any medium without royalty provided the copyright
# notice and this notice are preserved.  This file is offered as-is,
# without any warranty.
########################################################################

import pygame, sys
from pygame.locals import *
from screen import Screen, ORIGINAL_SCALE_AMOUNT
from opts import Options
from level import *
from sprites import *

FPS = 20

ZONE_MARKER_LIMIT = 0 # how high the zone marker can get before it's game over

#####
class Game:
    ###
    def __init__(self):
        pygame.init()
        self.screen = Screen()
        self.options = Options()
        self.clock = pygame.time.Clock()
        self.ticker = 0

    ###
    def playGame(self):
        level_num = self.options.starting_level
        self.playLevel(level_num)

    ###
    def initLevel(self, level_num):
        pygame.event.get()
        level = Level()
        self.screen.blit(level.background_image, \
                         level.background_image.get_rect())
        level.fill('black')
        return level

    ###
    def playLevel(self, level_num):
        # initialize level
        level = self.initLevel(level_num)

        # initialize borders
        left_border_s = BorderSprite(None, LEFT_BORDER_RECT)
        right_border_s = BorderSprite(None, RIGHT_BORDER_RECT)
        bottom_border_s = BorderSprite(None, BOTTOM_BORDER_RECT)
        left_border = Border(left_border_s)
        right_border = Border(right_border_s)
        bottom_border = Border(bottom_border_s, True)

        #cleared_rows = []
        forms = []
        while True:
            # clear the zone marker
            level.fill('black', zone_marker.rect)

            # create a new block when it's time
            if self.ticker % 75 == 0:
                forms.append(self.createFormation(-BLOCK_SIZE[Y], 3, 6))

            ## update the blocks
            self.clearRows()
            # move the blocks
            self.moveForms(forms, zone_marker, level)

            # draw the zone marker and make sure it's not too high
            zone_marker.setRect(clicked_group)
            level.fill('red', zone_marker.rect)
            if zone_marker.rect.top <= ZONE_MARKER_LIMIT:
                level.fill('black', zone_marker.rect)
                level.fill('red', zone_marker.rect)
                level.update(self.screen)
                self.gameOver()
                return ### SCORE WILL NEED TO BE RETURNED ONCE WE HAVE ONE


            # draw the screen
            level.update(self.screen)
            if self.ticker % 1 == 0:
                self.screen.update()


            # process the input
            for event in pygame.event.get():
                if event.type == KEYDOWN and event.key == ord('p'):
                    if raw_input() == 'q':
                        self.terminateGame()
                if event.type == KEYUP and event.key == ord('q'):
                    return
                if event.type == QUIT:
                    return
                if event.type == MOUSEBUTTONDOWN:
                    self.mouseClick(event.pos, self.screen)
                    pygame.mouse.get_rel()

                if event.type == MOUSEBUTTONUP:
                    self.mouseUnClick(event.pos, self.screen)

                if event.type == MOUSEMOTION and clicked_group != None:
                    clicked_group.mouse_pos = \
                        self.translateMousePos(event.pos, self.screen)

            # tick the clock
            self.clock.tick(FPS)
            self.ticker += 1
            if self.ticker == 10000:
                self.ticker = 0

    ###
    def mouseClick(self, screen_pos, screen):
        global clicked_group
        pos = self.translateMousePos(screen_pos, screen)
        for group in out_of_the_zone:
            for sprite in group.sprites():
                if sprite.rect.collidepoint(pos):
                    if type(group) == MultiBlock:
                        group.clicked += 1
        for group in in_the_zone:
            for sprite in group.sprites():
                if sprite.rect.collidepoint(pos):
                    clicked_group = group
                    clicked_group.at_bottom = False
                    #clicked_group.drag_limit = clicked_group.rect.top - \
                    #    BLOCK_SIZE[Y]
                    #print '1st', clicked_group
                    pygame.mouse.get_rel() # get an initial pos
                    if type(group) == MultiBlock:
                        if group.click_timer < FPS:
                            #group.clicked += 1
                            pass
                        else:
                            #group.clicked = 1
                            pass

    ###
    def mouseUnClick(self, screen_pos, screen):
        global clicked_group
        #pos = self.translateMousePos(screen_pos, screen)
        if clicked_group == None:
            return

        clicked_group.snap_to_grid_x_infected = True
        clicked_group.falling = True
        clicked_group.fall_speed = ZONE_MARKER_FALL_SPEED[:]
        clicked_group.at_bottom = False
        clicked_group = None
        #for group in in_the_zone:
        #    group.redraw_infected = True


    ###
    def translateMousePos(self, screen_pos, screen):
        pos = [X, Y]
        pos[X], pos[Y] = screen_pos[X], screen_pos[Y]
        if screen.window_size != screen.start_size:
            pos[X] /= ORIGINAL_SCALE_AMOUNT
            pos[Y] /= ORIGINAL_SCALE_AMOUNT
        pos[X] -= LEVEL_SURFACE_LEFT
        pos[Y] += LEVEL_SURFACE_TOP
        return pos
    ###
    def createFormation(self, top, size_min, size_max):
        color = random.randint(1, len(COLORS) - 1)
        left = random.randrange(0,499)
        x_speed = [random.randrange(2), random.randrange(2, 10)]
        if random.choice([True, False]):
            x_speed[SPEED] = -x_speed[SPEED]

        y_speed = [0,0]
        falling = True
        fall_speed = [2, 1]
        sprites = []
        for s in xrange(random.randint(size_min, size_max)):
            sprites.append(SimpleBlock(color))
        form = MultiBlock(sprites, left, top, x_speed, y_speed, fall_speed,
                          falling)
        form.rect = form.getRect()
        left = form.getLeft()
        right = form.getRight()
        if left < 0:
            form.moveSprites(-left+random.randrange(50),0)
            form.rect = form.getRect()
        if right > LEVEL_SURFACE_SIZE[X]:
            move_left = LEVEL_SURFACE_SIZE[X]-right-random.randrange(50)
            form.moveSprites(move_left, 0)
            form.rect = form.getRect()
        return form

    ###
    def clearRows(self):
        for row in LEVEL_Y_GRID:
            row_fill = 0
            for column in LEVEL_X_GRID:
                if level_grid[column, row]:
                    row_fill += 1
            if row_fill >= len(LEVEL_X_GRID):
                print 'Winning!'
                for column in LEVEL_X_GRID:
                    for group in level_grid[column, row].groups():
                        group.row_split_up = True
                        for group in in_the_zone:
                            group.fall_speed = zone_marker.fall_speed[:]
                            group.falling = True
                            group.at_bottom = False
                            group.redraw_infected = True
                    level_grid[column, row] = False

    ###
    def moveForms(self, forms, zone_marker, level):
        global clicked_group
        fill_rects = [] # used to clear a line after it's been filled
        for form in forms[:]:
            if form.click_timer == FPS:
                form.click_timer = 0

            if clicked_group == form:
                if form.row_split_up:
                    clicked_group = None

            if form.update(zone_marker, clicked_group) == SPLIT_UP \
                    or form.row_split_up:
                # split up a MultiBlock
                for sprite in form.sprites()[:]:
                    if not form.in_the_zone:
                        x_speed = [random.randrange(10,13),0]
                        y_speed = [random.randrange(6,9), 0]
                        if random.choice([True, False]):
                            x_speed[SPEED] = -x_speed[SPEED]
                        if random.choice([True, False]):
                            y_speed[SPEED] = -y_speed[SPEED]
                        fall_speed = [random.randrange(4,9), 0]
                    else: # in the zone
                        x_speed = form.x_speed[:]
                        y_speed = form.y_speed[:]
                        fall_speed = form.fall_speed[:]

                    # create the new sprite to replace the old one
                    newsprite = type(sprite)(GRAY)
                    newblock = SingleBlock(newsprite, sprite.rect.left,
                                           sprite.rect.top,
                                           x_speed, y_speed, fall_speed)
                    newblock.row_split_up = form.row_split_up
                    forms.append(newblock)
                    if not form.in_the_zone:
                        newblock.slow_down_x = True
                        newblock.slow_down_y = True
                        newblock.temp_non_fall = 0
                    else: # in the zone
                        in_the_zone.append(newblock)
                        # cause all of the in_the_zone to try to fall
                        for group in in_the_zone:
                            group.fall_speed = zone_marker.fall_speed[:]

                    if newblock.row_split_up:
                        for L in group_lists:
                            L.remove(newblock)
                        forms.remove(newblock)
                        fill_rects.append(newblock.getRect())
                        newblock.sprite.kill()

                forms.remove(form)
                collision_groups.remove(form)
                if form.in_the_zone:
                    # I don't know why I need this, but pyweek is almost over
                    # so let's see it it stops the random crash
                    try:
                        in_the_zone.remove(form)
                    except:
                        form.in_the_zone = False
                if not form.in_the_zone:
                    try:
                        out_of_the_zone.remove(form)
                    except:
                        pass # please don't crash.... :/

            for rect in fill_rects:
                level.fill('black', rect)
            if form.redraw:
                for rect in form.redraw_sprite_rects:
                    level.fill('black', rect)
                level.redrawn_rects.extend(form.draw(level.surface))
            form.click_timer += 1



    ###
    def gameOver(self):
        """
        Obviously, this needs to be expanded on.
        """
        self.screen.update()
        print 'Game Over'

    ###
    def terminateGame(self):
        pygame.quit()
        sys.exit()





